#ifndef RAWCONFIGURATIONSYSTEMIDV0_H
#define RAWCONFIGURATIONSYSTEMIDV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationSystemID
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconfsysid
#include "DAQDataFormats/DataFormatLast.inc"

// forward declarations
namespace daqdataformats{
    class RawConfigurationName;
}

namespace daqdataformats {

namespace  VERSION_NAMESPACE {
    namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t  HEADER_SIZE                = 1;//< Size of the Subsystem's header
            static const version_t Configuration_Name_Version = 0;//< Version of the ConfigurationName that this version of RawConfigurationSystemID uses.

            enum SysIDMASKS{
                MASK_Version        = 0x000000FF, //! Version number for the SystemID                                     (8 bits)
                MASK_DAQSystemID    = 0x000FFF00, //! Identification of the System(For instance DCMApp, BuffernodeEVBapp) (12 bits)
                MASK_DAQSubSystemID = 0xFFF00000, //! Identification of the Subsystem of the System (for instance DCMApp version, firmware version)(12 bits)
            };
            enum SysIDWORDS{
                WORD_Version        = 0, //! Version number for the SystemID                                     (8 bits)
                WORD_DAQSystemID    = 0, //! Identification of the System(For instance DCMApp, BuffernodeEVBapp) (12 bits)
                WORD_DAQSubSystemID = 0  //! Identification of the Subsystem of the System (for instance DCMApp version, firmware version)(12 bits)
            };

            DEFINE_SHIFT_FROM_MASK(Version)
            DEFINE_SHIFT_FROM_MASK(DAQSystemID)
            DEFINE_SHIFT_FROM_MASK(DAQSubSystemID)
    }// end of namespace rawconfsysid


  class RawConfigurationSystemID : public PREVIOUS_VERSION_CLASS_NAME {
    
    public:
            // Constructors
            RawConfigurationSystemID (const version_t version); //! The default constructor
            RawConfigurationSystemID (const RawConfigurationSystemID& copyin);
            /// Destructor
            ~RawConfigurationSystemID();

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const;
            bool     setBufferSource(const void*);

            using    PREVIOUS_VERSION_CLASS_NAME::readData; // unhide the readData methods
            void*    readData(const void*);

        protected:

            IMPLEMENT_FIRST_GET_AND_SET(Version       , uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(DAQSystemID   , uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(DAQSubSystemID, uint32_t);

            /// Set name
            INIT_FIRST(getVersionName, const, uint32_t,       std::string&);
            INIT_FIRST(setVersionName,      , uint32_t, const std::string&);

            INIT_FIRST(getHeaderSize , const, uint32_t, );

            /// Set an appropriate ConfigurationName Version,
            /// corresponding to the SystemID version
            bool setConfigurationNameVersion() {
                _configurationName_version = CURRENT_FIELDS_SUB_NAMESPACE::Configuration_Name_Version;
                return true;
            }

            ///
            uint32_t getPredefinedSize() const {return 2;}

        protected:
            RawConfigurationName*  _versionName;  //!< SubSystem's version name
            version_t              _configurationName_version;
  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONSYSTEMIDV0_H */
