#ifndef RAWCONFIGURATIONTAILV0_H
#define RAWCONFIGURATIONTAILV0_H
#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/Macros.h"
#include "DAQDataFormats/BitFields.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationTail
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconftail
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {
        // Configuration tail delimiter
        static const uint32_t Marker_value[2] = {0x464E4F43, 0x44444E45};
    }

namespace  VERSION_NAMESPACE {
    namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t RAWCONFIGURATIONTAIL_SIZE = 5;

            enum ConfigurationTailMASKS{
                MASK_Version   = 0x00000FFF, //! Configuration Block Tail Version (12 bits)
                MASK_Reserved1 = 0xFFFFF000, //! Reserved word 1                  (20 bits)
                MASK_Reserved2 = 0xFFFFFFFF, //! Reserved word 2                  (32 bits)
                MASK_MarkerLo1 = 0xFFFFFFFF, //! Marker lower word 1              (32 bits)
                MASK_MarkerLo2 = 0xFFFFFFFF, //! Marker lower word 2              (32 bits)
                MASK_CRC       = 0xFFFFFFFF  //! Check sum for the tail           (32 bits)
            };

            enum ConfigurationTailWORDS{
                WORD_Version   = 0, //! Version
                WORD_Reserved1 = 0, //! Reserved word
                WORD_Reserved2 = 1, //! Reserved word
                WORD_MarkerLo1 = 2, //! Marker lower word 1      (32 bits)
                WORD_MarkerLo2 = 3, //! Marker lower word 2      (32 bits)
                WORD_CRC       = 4  //! Check sum for the tail   (32 bits)
            };

            DEFINE_SHIFT_FROM_MASK(Version);
            DEFINE_SHIFT_FROM_MASK(Reserved1);
            DEFINE_SHIFT_FROM_MASK(Reserved2);
            DEFINE_SHIFT_FROM_MASK(MarkerLo1);
            DEFINE_SHIFT_FROM_MASK(MarkerLo2);
            DEFINE_SHIFT_FROM_MASK(CRC);
    }// end of namespace rawconftail


  class RawConfigurationTail : public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
            RawConfigurationTail (const version_t); ///< The default constructor
            RawConfigurationTail (const RawConfigurationTail&);
            ~RawConfigurationTail(); ///< Destructor

            void     init            ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print           (std::ostream& os=std::cout) const;
            uint32_t sizeofdata      () const {return CURRENT_FIELDS_SUB_NAMESPACE::RAWCONFIGURATIONTAIL_SIZE;}
            using    PREVIOUS_VERSION_CLASS_NAME::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            /// Make set protected, so that users of the class cannot access this method
            IMPLEMENT_FIRST_GET_AND_SET(Version  , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Reserved1, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(Reserved2, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(MarkerLo1, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(MarkerLo2, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(CRC      , uint32_t)
  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* RAWCONFIGURATIONTAILV0_H */
