#ifndef DAQRAWTRIGGERTIMEV0_H
#define DAQRAWTRIGGERTIMEV0_H

#include "DAQDataFormats/RawDAQData.h"
#include "DAQDataFormats/BitFields.h"
#include "DAQDataFormats/TriggerDefines.h"
#include "DAQDataFormats/Macros.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerTime
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggertime
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats {
namespace VERSION_NAMESPACE {

    namespace CURRENT_FIELDS_SUB_NAMESPACE {

            static const uint32_t TRIGGERTIME_SIZE = 5;

            enum TriggerTimeMASKS{
                MASK_TriggerTimeStampLow    = 0xFFFFFFFF,
                MASK_TriggerTimeStampHigh   = 0x00FFFFFF,
                MASK_Version                = 0xFF000000,
                MASK_TriggerTimeStamp_sec   = 0xFFFFFFFF,
                MASK_TriggerTimeStamp_usec  = 0xFFFFFFFF,
                MASK_ClockSource            = 0x000000FF
            };

            enum TriggerTimeWORDS{
                WORD_TriggerTimeStampLow    = 0,
                WORD_TriggerTimeStampHigh   = 1,
                WORD_Version                = 1,
                WORD_TriggerTimeStamp_sec   = 2,
                WORD_TriggerTimeStamp_usec  = 3,
                WORD_ClockSource            = 4
            };

            DEFINE_SHIFT_FROM_MASK(TriggerTimeStampLow)
            DEFINE_SHIFT_FROM_MASK(TriggerTimeStampHigh)
            DEFINE_SHIFT_FROM_MASK(Version)
            DEFINE_SHIFT_FROM_MASK(TriggerTimeStamp_sec)
            DEFINE_SHIFT_FROM_MASK(TriggerTimeStamp_usec)
            DEFINE_SHIFT_FROM_MASK(ClockSource)
  }// end of namespace


  //! Class to hold the data corresponding to the generated time of the trigger
  /*!
    The RawTriggerTimeV0 contains the information pertaining to the time at
    which the trigger was generated, and what device generated the timestamp.

    The class supports general initialization with an internal or external
    data buffer as well as copyin style initialization for use with
    STL containers.
  */
  class RawTriggerTime: public PREVIOUS_VERSION_CLASS_NAME {
    
        public:
      
            RawTriggerTime (const version_t); ///< The default constructor
            RawTriggerTime (const RawTriggerTime&);
            ~RawTriggerTime(); ///< Destructor

            void     init      ();
            using    PREVIOUS_VERSION_CLASS_NAME::print;
            void     print     (std::ostream& os=std::cout) const;
            uint32_t sizeofdata() const { return CURRENT_FIELDS_SUB_NAMESPACE::TRIGGERTIME_SIZE;}
            /// Nanoslice of version 0 is a fixed size format
            using    RawDAQData::readData;
            void*    readData(const void* pointer){return RawDAQData::readDataGeneral(pointer);};

            IMPLEMENT_FIRST_GET_AND_SET(Version      , uint32_t);
            IMPLEMENT_FIRST_GET_AND_SET(TriggerTimeStampLow  , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerTimeStampHigh , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerTimeStamp_sec , uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(TriggerTimeStamp_usec, uint32_t)
            IMPLEMENT_FIRST_GET_AND_SET(ClockSource          , uint32_t)

	    INIT_FIRST(getTriggerTimeStamp      , const, uint64_t, )
            INIT_FIRST(setTriggerTimeStamp      ,      , bool   , const uint64_t)
	    INIT_FIRST(getTriggerTimeStampTvUsec, const, suseconds_t, )
            INIT_FIRST(setTriggerTimeStampTvUsec,      , bool   , const suseconds_t)
	    INIT_FIRST(getTriggerTimeStampTvSec , const, time_t, )
            INIT_FIRST(setTriggerTimeStampTvSec ,      , bool   , const time_t)
	    INIT_FIRST(getTriggerTimeStampTv    , const, timeval, )
            INIT_FIRST(setTriggerTimeStampTv    ,      , bool   , const timeval)
  };
  
  
}} // END NAMESPCE daqdataformats
#endif /* DAQRAWTRIGGERTIMEV0_H */
