#ifndef _DBUTILS_DAQAPPMGR_CONFIGUREDPROCESSLIST_H
#define _DBUTILS_DAQAPPMGR_CONFIGUREDPROCESSLIST_H

#include <DatabaseUtils/DAQAppMgr/ApplicationInstance.h>
#include <boost/shared_ptr.hpp>
#include <string>
#include <vector>

namespace dbutils {
namespace daqappmgr {

/**
 * Container class for the full set of processes that should be
 * run in the DAQ system.
 *
 * @author Kurt Biery
 * @version $Revision: 1.3 $ $Date: 2012/05/30 14:27:44 $
 */

class ConfiguredProcessList {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    ConfiguredProcessList(const std::string& detectorName,
                          int partitionNumber = 0,
                          const bool& useDBData = false);

    // *****************
    // *** Accessors ***
    // *****************

    bool hasValidData() const {return _hasValidData;}

    std::vector<std::string> getUniqueAppTypes() const {return _uniqueAppTypes;}
    std::vector<std::string> getUniqueHosts() const {return _uniqueHosts;}

    bool getProcessListByType(const std::string& appType,
                              std::vector<ApplicationInstance>&
                              resultingAppList) const;
    bool getProcessListByExcludedType(const std::vector<std::string>&
                                      appTypeList,
                                      std::vector<ApplicationInstance>&
                                      resultingAppList) const;
    boost::shared_ptr<ApplicationInstance>
        getProcess(const std::string& appType,
                   const std::string& appName) const;

    // temporary modifier

    void _addApplicationInstance(const ApplicationInstance& appInstance);

    // *****************
    // *** Operators ***
    // *****************

    bool operator==(const ConfiguredProcessList& other);
    bool operator!=(const ConfiguredProcessList& other);

 private:

    bool _fetchProcessData(const std::string& tableDefinitionFilename,
                           const std::string& detectorName,
                           const std::string& inputAppType);

    bool _hasValidData;
    std::vector<std::string> _uniqueAppTypes;
    std::vector<std::string> _uniqueHosts;

    std::vector<ApplicationInstance> _fullProcessList;
};

};  // end of namespace daqappmgr
};  // end of namespace dbutils

#endif
