#ifndef _DBUTILS_DAQCONFIG_GTRUNCONFIG_H
#define _DBUTILS_DAQCONFIG_GTRUNCONFIG_H

#include <DatabaseUtils/SimpleParameterSet.h>
#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <map>
#include <string>
#include <vector>
#include <stdexcept>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching of Global Trigger
 * run configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.9 $ $Date: 2015/06/28 13:14:22 $
 */
class GTRunConfig {

 public:
    //exception class with error msg to be thrown on DB reading error
    class db_exception: public std::runtime_error{
        public:
          db_exception(std::string Where, std::string What):
              std::runtime_error("ERROR in <"+Where+">: "+What+"!"){};
   };

 public:

    typedef std::vector<boost::shared_ptr<SimpleParameterSet> > TriggerList;

    // ********************
    // *** Constructors ***
    // ********************

    GTRunConfig(const std::vector<int64_t>& subsystemConfigIdList);

    GTRunConfig(const SubsystemNameSpec& nameSpec);

    GTRunConfig(const GlobalNameSpec& nameSpec);

    GTRunConfig(const int64_t& globalConfigId);

    virtual ~GTRunConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}

    TriggerList getCalibrationTriggers() {return _calibrationTriggerList;}
    TriggerList getExternalTriggers() {return _externalTriggerList;}
    TriggerList getManualTriggers() {return _manualTriggerList;}
    TriggerList getSpillTriggers() {return _spillTriggerList;}
    TriggerList getDataDrivenTriggers() {return _dataDrivenTriggerList;}
    TriggerList getSNEWSTriggers() {return _SNEWSTriggerList;}

    boost::shared_ptr<SimpleParameterSet> getGlobalThrottle() {
        return _globalThrottle;
    }

    boost::shared_ptr<SimpleParameterSet> getDDTThrottle() {
         return _DDTThrottle;
    }

    boost::shared_ptr<SimpleParameterSet> getTriggerOffset() {
        return _triggerOffset;
    }

 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList);

    bool _fetchParameterSet(const std::string& tableDefinitionFileName,
                           const int64_t& subsystemConfigId,
                           boost::shared_ptr<SimpleParameterSet> & parSet);

    bool _fetchTriggerData(const std::string& tableDefinitionFileName,
                           const int64_t& subsystemConfigId,
                           TriggerList& triggerList);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    TriggerList _calibrationTriggerList;
    TriggerList _externalTriggerList;
    TriggerList _manualTriggerList;
    TriggerList _spillTriggerList;
    TriggerList _dataDrivenTriggerList;
    TriggerList _SNEWSTriggerList;
    TriggerList _NSNFilterList;
    boost::shared_ptr<SimpleParameterSet> _NSNTriggerParams;
    boost::shared_ptr<SimpleParameterSet> _globalThrottle;
    boost::shared_ptr<SimpleParameterSet> _DDTThrottle;
    boost::shared_ptr<SimpleParameterSet> _triggerOffset;
};

 

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
