#include <sys/types.h>		// Standard System Types
#include <stdlib.h>		// Standard Library Header
#include <stdio.h>		// 
#include <time.h>		// Standard System Time Header
#include "DAQDataFormats/DCMSimulator.h"

#define DEFAULT_HIT_PROBABILITY 0.1
#define randReal() (1.0*rand() )/RAND_MAX

using namespace daqdataformats;

DCMSimulator::DCMSimulator(){
  this->init();
}

DCMSimulator::DCMSimulator(uint32_t diblock_number, uint32_t dcm_number){
  this->init();
  this->dcmID     = dcm_number;
  this->diblockID = diblock_number;
}


DCMSimulator::~DCMSimulator(){
  
}

DCMSimulator::DCMSimulator(uint32_t FEBMask){
  this->setFEBMask(FEBMask);
  this->setHitProbability(DEFAULT_HIT_PROBABILITY);	// Default hit probability
}

DCMSimulator::DCMSimulator(uint32_t FEBMask, float probability){
  this->setFEBMask(FEBMask);
  this->setHitProbability(probability);	// Default hit probability
}

bool DCMSimulator::init(){
  this->dcmID=0;		// Unassigned
  this->diblockID=0;		// Unassigned
  this->clock=0;		// Reset the clock to zero time
  this->febMask=0x0;		// All FEBs are off by default
  this->setHitProbability(DEFAULT_HIT_PROBABILITY);	// Default hit probability

  // Clear the databuffer
  int i=0;
  while(i < DCM_DATA_BUFFER_SIZE){
    this->databuffer[i]=0x0;
    ++i;
  }

  this->nextWord = &(databuffer[0]);

  i=0;
  while(i < 64){
    ActiveFEBList[i]=NULL;
    ++i;
  }

  return true;
}


bool DCMSimulator::initFEB(int FEBNumber){
    return true;
}

bool DCMSimulator::initFEBList(){
  // We look at the FEB Mask and make a list of the pointers to the 
  // active FEBs for subsequent readouts

  int j=0;
  for(int i=0;i<64;++i){
    if(febMask & (0x1 << i)){
      ActiveFEBList[j] = &(FrontEndBoards[i]);
      ++j;
    } // endif febMask
  } // endfor i

  return true;
}

void DCMSimulator::setFEBMask(uint32_t FEBMask){
  febMask = FEBMask;	// Set the front end board mask
}

void DCMSimulator::setHitProbability(float probability){
  hit_probability = probability;	// Set the prob of generating a single hit
}


RawMicroSlice* DCMSimulator::generateMicroSlice(){
  // We need to generate a microslice
  // from scratch.  We do this in stages
  
  // First generate the header
  RawMicroSliceHeader microHeader;
  //microHeader.setDataPresent(true);

  // Note: The header is generated with a zero byte count
  // so we need to set that later when we know the size of the block

  // Now generate the time marker;
  RawTimingMarker tMarker;
  tMarker.generate();

  

  return &LastMicroSlice;
}

#ifdef ASDF

const RawNanoSlice& DCMSimulator::GenerateNanoSlice(int PixelID){
  NanoSliceHeader FullHeader;
  uint32_t EncodedHeader;
  int TimeStamp;
  // time_t TimeStamp;
  uint32_t PulseHeight;

  // Fill the header information for the nano slice
  FullHeader.FrameSequenceNumber = SEQ;
  FullHeader.PixelAddress = PixelID & 0x001F;  // Generate a random Pixel
  FullHeader.FEBStatus = 0x1;
  FullHeader.FEBLinkID = FEBLinkID;
  FullHeader.DebugMode = false;
  FullHeader.DataPresent = false;
  FullHeader.LinkStatus = true;

  // Check to see if we should generate a hit for this block
  if( randReal() < this->HitProbability){
    //    printf("%2.6f\n",randReal());
    
    FullHeader.DataPresent = true;
    
    // Generate the time stamp (just from the current time)
    TimeStamp = time((time_t*)&TimeStamp);
    // TimeStamp = 0xFFFFFFFF;

    // Generate a random pulse height (12bit)
    PulseHeight = rand() & 0x00000FFF;
    // Shift the pulse height over to give the right word alignment;
    PulseHeight = PulseHeight << 16;

    // Now Advance the Sequence Number
    this->AdvanceSEQ();

  }else{
    TimeStamp = 0x0;
    PulseHeight = 0x0;
  }

  EncodedHeader = LastNanoSlice.PackNanoSliceHeader(FullHeader);

  LastNanoSlice.SetHeader(EncodedHeader);
  LastNanoSlice.SetTimeStamp(TimeStamp);
  LastNanoSlice.SetPulseHeight(PulseHeight);
}

// This version forces hit generation of a hit on a given pixel
const RawNanoSlice& DCMSimulator::GenerateNanoSlice(int PixelID, bool ForceGeneration){
  NanoSliceHeader FullHeader;
  uint32_t EncodedHeader;
  uint32_t PulseHeight;
  int TimeStamp;

  // Fill the header information for the nano slice
  FullHeader.FrameSequenceNumber = SEQ;
  FullHeader.PixelAddress = PixelID & 0x001F;  // Generate a random Pixel
  FullHeader.FEBStatus = 0x1;
  FullHeader.FEBLinkID = FEBLinkID;
  FullHeader.DebugMode = false;
  FullHeader.DataPresent = false;
  FullHeader.LinkStatus = true;    
  FullHeader.DataPresent = true;
    
  // Generate the time stamp (just from the current time)
  TimeStamp = time((time_t*)&TimeStamp);
  // TimeStamp = 0xFFFFFFFF;
  
  // Generate a random pulse height (12bit)
  PulseHeight = rand() & 0x00000FFF;
  // Shift the pulse height over to give the right word alignment;
  PulseHeight = PulseHeight << 16;
  
  // Now Advance the Sequence Number
  this->AdvanceSEQ();

  // Encode the data into the block
  EncodedHeader = LastNanoSlice.PackNanoSliceHeader(FullHeader);

  // Set the block data
  LastNanoSlice.SetHeader(EncodedHeader);
  LastNanoSlice.SetTimeStamp(TimeStamp);
  LastNanoSlice.SetPulseHeight(PulseHeight);
}

// This version forces hit generation of a hit on a given pixel
const RawNanoSlice& DCMSimulator::GenerateNanoSlice(int PixelID,int iTimeStamp, int iPulseHeight, bool ForceGeneration){
  NanoSliceHeader FullHeader;
  uint32_t EncodedHeader;
  uint32_t PulseHeight;
  int TimeStamp;

  // Fill the header information for the nano slice
  FullHeader.FrameSequenceNumber = SEQ;
  FullHeader.PixelAddress = PixelID & 0x001F;  // Generate a random Pixel
  FullHeader.FEBStatus = 0x1;
  FullHeader.FEBLinkID = FEBLinkID;
  FullHeader.DebugMode = false;
  FullHeader.DataPresent = false;
  FullHeader.LinkStatus = true;    
  FullHeader.DataPresent = true;
    
  // Set the timestamp
  TimeStamp = iTimeStamp;
  
  // Set the PulseHeight (and limit it to 12 bits with an overflow
  if(iPulseHeight & 0xFFFFF000){iPulseHeight= 0x00000FFF;}
  PulseHeight = iPulseHeight & 0x00000FFF;
  // Shift the pulse height over to give the right word alignment;
  PulseHeight = PulseHeight << 16;
  
  // Now Advance the Sequence Number
  this->AdvanceSEQ();

  // Encode the data into the block
  EncodedHeader = LastNanoSlice.PackNanoSliceHeader(FullHeader);

  // Set the block data
  LastNanoSlice.SetHeader(EncodedHeader);
  LastNanoSlice.SetTimeStamp(TimeStamp);
  LastNanoSlice.SetPulseHeight(PulseHeight);
}


bool DCMSimulator::nonEmptyNanoSlice(){
  if(LastNanoSlice.getNanoSliceHeader() & 0x2){return true;}
  return false;
}

#endif

void DCMSimulator::printMicroSlice(){

#ifdef ASDF
  uint32_t rawheader;
  NanoSliceHeader header;

  printf("Header: 0x%08x \t Time: 0x%08x\t PulseHeight: 0x%08x\t",LastNanoSlice.GetNanoSliceHeader(), LastNanoSlice.GetTimeStamp(), LastNanoSlice.GetPulseHeight());

  rawheader = LastNanoSlice.GetNanoSliceHeader();
  
  LastNanoSlice.UnPackNanoSliceHeader(rawheader, &header);
  
  printf("Frame: %d\tPixel: %d\tStatus: %d\tLink: %d\n",header.FrameSequenceNumber, header.PixelAddress, header.FEBStatus, header.FEBLinkID);

#endif
}
