#include "DAQDataFormats/RawConfigurationBlock.h"
#include "DAQDataFormats/RawConfigurationTail.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationBlock
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE configurationblock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{

//------------------------------------------------------------------------------
// Default constructor
RawConfigurationBlock::RawConfigurationBlock(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    reserveInternalBuffer(configurationblock::ConfigurationBlockMaximumSize);
    defaultConstructorFloatingSize();

    _tail = new RawConfigurationTail(_version_ConfigurationTail);
}

//------------------------------------------------------------------------------
RawConfigurationBlock::RawConfigurationBlock(const RawConfigurationBlock& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    lastOperationsInDeepCopy(copy_in);
    _tail = new RawConfigurationTail(_version_ConfigurationTail);
}

//------------------------------------------------------------------------------
RawConfigurationBlock::RawConfigurationBlock(const void* datablock_buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    reserveInternalBuffer(configurationblock::ConfigurationBlockMaximumSize);
    copy(datablock_buffer);

    _tail = new RawConfigurationTail(_version_ConfigurationTail);
}

//------------------------------------------------------------------------------
// Destructor
RawConfigurationBlock::~RawConfigurationBlock() {}


//------------------------------------------------------------------------------
/// All binding routines
#define STANDARD_ROUTINES                                                                              \
            BIND_CUSTOM_FUNCTION(close                            , version0,      , bool, const bool);\
            BIND_CUSTOM_FUNCTION(getHeader                        , version0,      , daqdataformats::RawConfigurationHeader*, ); \
            BIND_CUSTOM_FUNCTION(getTail                          , version0,      , daqdataformats::RawConfigurationTail*, ); \
            BIND_CUSTOM_FUNCTION(getTailConst                     , version0, const, daqdataformats::RawConfigurationTail*, ); \
            BIND_CUSTOM_FUNCTION(getSystemID                      , version0,      , daqdataformats::RawConfigurationSystemID*, ); \
            BIND_CUSTOM_FUNCTION(getSystemIDConst                 , version0, const, daqdataformats::RawConfigurationSystemID*, ); \
            BIND_CUSTOM_FUNCTION(getSystemIDWithInput             , version0,      , daqdataformats::RawConfigurationSystemID*, const uint32_t);\
            BIND_CUSTOM_FUNCTION(getSystemIDConstWithInput        , version0, const, daqdataformats::RawConfigurationSystemID*, const uint32_t);\
            BIND_CUSTOM_FUNCTION(setFloatingSystemIDToBufferSource, version0, const, bool, const void*);\
            BIND_CUSTOM_FUNCTION(setFloatingSystemID              , version0, const, bool, const uint32_t);\
            BIND_CUSTOM_FUNCTION(addSystemID                      , version0,      , bool, const void*);\
            BIND_CUSTOM_FUNCTION(getMarkerHi1                     , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getMarkerHi2             	  , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getMarkerLo1             	  , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getMarkerLo2             	  , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getConfigurationVersion  	  , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getConfigurationBlockSize        , version0, const, uint32_t, ); \
            BIND_CUSTOM_FUNCTION(getSystemNumber                  , version0, const, uint32_t, ); \
            BIND_GET_AND_SET(DetectorID       , version0, uint32_t);\
            BIND_GET_AND_SET(ResourceManagerID, version0, uint32_t);\
            BIND_GET_AND_SET(RunNumber        , version0, uint32_t);\
            BIND_GET_AND_SET(RunType          , version0, uint32_t);\
            BIND_GET_AND_SET(Partition        , version0, uint32_t);\
            BIND_GET_AND_SET(Subrun           , version0, uint32_t);\
            BIND_GET_AND_SET(SystemVersionIDX , version0, uint32_t);\
            BIND_STADARD_FUNCTIONS;                                 \
            CURRENT_VERSION_NAMESPACE::CURRENT_CLASS::setVersions();

#include "DAQDataFormats/setFunctionPointers_template.inc"

}// end of namespaces
