#include "DAQDataFormats/RawEventTailV0.h"
#include <boost/format.hpp>
/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawEventTail
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE raweventtail
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------------------------------------------------------------
// Default constructor
RawEventTail::RawEventTail(const version_t version):
             RawDAQData(version)
            ,ZEROOUT_GET_AND_SET(Marker       )
            ,ZEROOUT_GET_AND_SET(MarkerLeft   )
            ,ZEROOUT_GET_AND_SET(MarkerRight  )
            ,ZEROOUT_GET_AND_SET(Version      )
            ,ZEROOUT_GET_AND_SET(EventNumberLo)
            ,ZEROOUT_GET_AND_SET(EventNumberHi)
            ,ZEROOUT_GET_AND_SET(EventSize    )
{ }

//------------------------------------------------------------------------------
RawEventTail::RawEventTail(const RawEventTail& copyin):
             RawDAQData(copyin)
            ,DEEPCOPY_GET_AND_SET(Marker       )
            ,DEEPCOPY_GET_AND_SET(MarkerLeft   )
            ,DEEPCOPY_GET_AND_SET(MarkerRight  )
            ,DEEPCOPY_GET_AND_SET(Version      )
            ,DEEPCOPY_GET_AND_SET(EventNumberLo)
            ,DEEPCOPY_GET_AND_SET(EventNumberHi)
            ,DEEPCOPY_GET_AND_SET(EventSize    )
{ }

//------------------------------------------------------------------------------
// Destructor
RawEventTail::~RawEventTail() { }

//------------------------------------------------------------------------------
// Initialization of data members
void RawEventTail::init(){

    EXECUTE_FUNCTION(setVersion    )(_version);
    EXECUTE_FUNCTION(setMarker     )(daqdataformats::CURRENT_FIELDS_SUB_NAMESPACE::Marker_value[0]);
    EXECUTE_FUNCTION(setMarkerLeft )(daqdataformats::CURRENT_FIELDS_SUB_NAMESPACE::Marker_value[1]);
    EXECUTE_FUNCTION(setMarkerRight)(daqdataformats::CURRENT_FIELDS_SUB_NAMESPACE::Marker_value[2]);
}
//------------------------------------------------------------------------------
// Print the RawEventTail header information
void RawEventTail::print(std::ostream& os) const{
    os<<boost::format("\nRawEvent Tail version %d :") % EXECUTE_FUNCTION(getVersion)();
    os<<boost::format(" Marker=%u"         ) % EXECUTE_FUNCTION(getMarker)();       //! RawEventTail header delimiter
    os<<boost::format(" EventNumberLo=%u"  ) % EXECUTE_FUNCTION(getEventNumberLo)();
    os<<boost::format(" EventNumberHi=%u"  ) % EXECUTE_FUNCTION(getEventNumberHi)();
    os<<boost::format(" MarkerLeft=%u"     ) % EXECUTE_FUNCTION(getMarkerLeft)();       //! RawEventTail header delimiter
    os<<boost::format(" MarkerRight=%u"    ) % EXECUTE_FUNCTION(getMarkerRight)();       //! RawEventTail header delimiter
    os<<boost::format(" EventSize=%u"      ) % EXECUTE_FUNCTION(getEventSize)();

    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<"\nRAW:";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<" ";
        printWord(i,false, os);
    }

    os<<"\nRAW:";
    for(uint32_t i = 0; i < size_of_data; ++i){
        os<<std::endl;
        printWord(i,true, os);
    }

}


}}